/*
  Macro           GenWordSet
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Compatibility   TSE v4 upwards
  Version         v1.1.1   6 Apr 2023

  This interactive tool generates a TSE WordSet based on the characters
  that it finds in the current buffer.

  You can copy the generated WordSet from the result-prompt.


  INSTALLATION

  Just copy this file to TSE's "mac" directory and compile it there,
  for example by opening the file there and applying the Macro Compile menu.


  USAGE

  For example execute "GenWoprdSet" from the Macro Execute menu.

  This tool works based on the whole current buffer.
  If you want to base it on a part of a buffer or file, then you first need to
  copy that part to a new empty buffer.

  A buffer usually contains space characters, but for most purposes WordSets
  should not include them.
  Therefore, if the tool encounters a space character, it will ask to omit it
  from the WordSet. The default answer is Yes.

  The tool will use the range character "-" to generate the shortest possible
  text string for the WordSet.
  In TSE's "ASCII Chart ..." menu you can see which characters are between a
  range's start and end character.

  An ordinary TSE text buffer will not contain carriage return or line feed
  characters, so those will ordinarily not be part of the WordSet.

  TSE's ChrSet() documentation contains more complete info than its WordSet
  documentation.


  HISTORY

  v1      5 Apr 2023
    Initial release.

  v1.1    5 Apr 2023
    Usually the space character needs to be omitted from the intended WordSet.
    Added a question that default does that.

  v1.1.1  6 Apr 2023
    Bug: If a file contained a backslash character, then the WordSet could miss
    an unrelated character.

*/



// This proc does the opposite of TSE's ChrSet() function.
string proc decode_wordset(string bits)
  string  char             [1] = ''
  string  chars [MAXSTRINGLEN] = ''
  integer i                    = 0
  integer len                  = 0
  integer omit_space_yes_no    = 0

  for i = 0 to 255
    if GetBit(bits, i)
      char = Chr(i)
      case char
        when ' '
          if not omit_space_yes_no
            omit_space_yes_no =
              MsgBox('Omit space characters from WordSet?',
                     'Space characters were found.'
                     + Chr(13)
                     + 'They usually need to be omitted from the generated WordSet.'
                     + Chr(13)
                     + 'Omit?',
                     _YES_NO_)
            if not omit_space_yes_no
              omit_space_yes_no = 1
              Message('<Escape> means Yes: Space characters are omitted from the WordSet.')
              Delay(9) // Improves user experience.
            endif
          endif
          if omit_space_yes_no <> 1
            chars = chars + ' '
          endif
        when '-'
          chars = '-' + chars
        when '\'
          chars = chars + '\\'
        otherwise
          chars = chars + char
          len   = Length(chars)

          if  len >= 4
          and Pos('\', chars[len - 3: 4]) == 0
            if  Pos('-', chars[len - 3: 4]) == 0
            and Asc(chars[len   ]) == Asc(chars[len - 1]) + 1
            and Asc(chars[len -1]) == Asc(chars[len - 2]) + 1
            and Asc(chars[len -2]) == Asc(chars[len - 3]) + 1
              chars = chars[1 .. len - 3] + '-' + chars[len]
            else
              if  chars[len - 1] <> '-'
              and chars[len - 2] == '-'
              and chars[len - 3] <> '-'
              and Asc(chars[len]) == Asc(chars[len - 1]) + 1
                chars[len - 1] = chars[len]
                chars          = chars[1: len - 1]
              endif
            endif
          endif
      endcase
    endif
  endfor

  i = 1
  while i <= Length(chars)
    if (Asc(chars[i]) in 0 .. 31, 255)
      chars = Format(chars[1: i - 1], '\d', Asc(chars[i]):3:'0',
                     chars[i + 1: MAXSTRINGLEN])
      i     = i + 4
    endif
    i = i + 1
  endwhile

  return(chars)
end decode_wordset


proc Main()
  integer c                           = 0
  string  wordset_bits           [32] = ChrSet('')
  string  wordset_text [MAXSTRINGLEN] = ''
  PushPosition()
  BegFile()
  repeat
    c = CurrChar()
    if not (c in _AT_EOL_, _BEYOND_EOL_)
      SetBit(wordset_bits, c)
    endif
  until not NextChar()
  PopPosition()
  wordset_text = decode_wordset(wordset_bits)
  Ask('Copy the found WordSet from here:', wordset_text)
  PurgeMacro(CurrMacroFilename())
end Main

